@@ -1,8 +1,6 @@
 # Data::Compare - compare perl data structures
 # Author: Fabien Tassin <fta@sofaraway.org>
 # updated by David Cantrell <david@cantrell.org.uk>
-# Copyright 1999-2001 Fabien Tassin <fta@sofaraway.org>
-# portions Copyright 2003 - 2013 David Cantrell
 
 package Data::Compare;
 
@@ -12,12 +10,13 @@ use warnings;
 use vars qw(@ISA @EXPORT $VERSION $DEBUG %been_there);
 use Exporter;
 use Carp;
+use Clone qw(clone);
 use Scalar::Util qw(tainted);
 use File::Find::Rule;
 
 @ISA     = qw(Exporter);
 @EXPORT  = qw(Compare);
-$VERSION = 1.26;
+$VERSION = 1.28;
 $DEBUG   = $ENV{PERL_DATA_COMPARE_DEBUG} || 0;
 
 my %handler;
@@ -99,7 +98,14 @@ sub Compare {
 
   my $x = shift;
   my $y = shift;
-  my $opts = shift || {};
+  my $opts = {};
+  if(@_) { $opts = clone(shift); }
+
+  _Compare($x, $y, $opts);
+}
+
+sub _Compare {
+  my($x, $y, $opts) = @_;
   my($xparent, $xpos, $yparent, $ypos) = map {
     $opts->{$_} || ''
   } qw(xparent xpos yparent ypos);
@@ -113,7 +119,7 @@ sub Compare {
   $opts->{recursion_detector}++;
 
   warn "Yaroo! deep recursion!\n" if($opts->{recursion_detector} == 99);
-  
+
   if(
     (ref($x) && exists($been_there{"$x-$xpos-$xparent"}) && $been_there{"$x-$xpos-$xparent"} > 1) ||
     (ref($y) && exists($been_there{"$y-$ypos-$yparent"}) && $been_there{"$y-$ypos-$yparent"} > 1)
@@ -149,7 +155,7 @@ sub Compare {
       $rval = 1;
     }
     elsif ($refx eq 'SCALAR' || $refx eq 'REF') {
-      $rval = Compare(${$x}, ${$y}, $opts);
+      $rval = _Compare(${$x}, ${$y}, $opts);
     }
     elsif ($refx eq 'ARRAY') {
       if ($#{$x} == $#{$y}) { # same length
@@ -157,7 +163,7 @@ sub Compare {
         $rval = 1;
         for (@$x) {
           $i++;
-          $rval = 0 unless Compare($x->[$i], $y->[$i], { %{$opts}, xparent => $x, xpos => $i, yparent => $y, ypos => $i});
+          $rval = 0 unless _Compare($x->[$i], $y->[$i], { %{$opts}, xparent => $x, xpos => $i, yparent => $y, ypos => $i});
         }
       }
       else {
@@ -171,12 +177,15 @@ sub Compare {
       $rval = 0 unless scalar @kx == scalar @ky;
 
       for (@kx) {
-        next unless defined $x->{$_} || defined $y->{$_};
-        $rval = 0 unless defined $y->{$_} && Compare($x->{$_}, $y->{$_}, { %{$opts}, xparent => $x, xpos => $_, yparent => $y, ypos => $_});
+        if(!exists($y->{$_})) {
+            $rval = 0;
+            last;
+        }
+        $rval = 0 unless _Compare($x->{$_}, $y->{$_}, { %{$opts}, xparent => $x, xpos => $_, yparent => $y, ypos => $_});
       }
     }
     elsif($refx eq 'Regexp') {
-      $rval = Compare($x.'', $y.'', $opts);
+      $rval = _Compare($x.'', $y.'', $opts);
     }
     elsif ($refx eq 'CODE') {
       $rval = 0;
@@ -189,21 +198,21 @@ sub Compare {
       if ($type eq 'HASH') {
         my %x = %$x;
         my %y = %$y;
-        $rval = Compare(\%x, \%y, { %{$opts}, xparent => $xparent, xpos => $xpos, yparent => $yparent, ypos => $ypos});
+        $rval = _Compare(\%x, \%y, { %{$opts}, xparent => $xparent, xpos => $xpos, yparent => $yparent, ypos => $ypos});
         $been_there{\%x."-$xpos-$xparent"}--; # decrement count for temp structures
         $been_there{\%y."-$ypos-$yparent"}--;
       }
       elsif ($type eq 'ARRAY') {
         my @x = @$x;
         my @y = @$y;
-        $rval = Compare(\@x, \@y, { %{$opts}, xparent => $xparent, xpos => $xpos, yparent => $yparent, ypos => $ypos});
+        $rval = _Compare(\@x, \@y, { %{$opts}, xparent => $xparent, xpos => $xpos, yparent => $yparent, ypos => $ypos});
         $been_there{\@x."-$xpos-$xparent"}--;
         $been_there{\@y."-$ypos-$yparent"}--;
       }
       elsif ($type eq 'SCALAR' || $type eq 'REF') {
         my $x = ${$x};
         my $y = ${$y};
-        $rval = Compare($x, $y, $opts);
+        $rval = _Compare($x, $y, $opts);
         # $been_there{\$x}--;
         # $been_there{\$y}--;
       }
@@ -408,7 +417,7 @@ Copyright (c) 1999-2001 Fabien Tassin. All rights reserved.
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
 
-Some parts copyright 2003 - 2014 David Cantrell.
+Some parts copyright 2003 - 2023 David Cantrell.
 
 Seeing that Fabien seems to have disappeared, David Cantrell has become
 a co-maintainer so he can apply needed patches.  The licence, of course,
